@@ -385,31 +385,68 @@ Emulate only the body of the API Gateway event.
     return fs.existsSync(path.join(codeDirectory, 'package-lock.json'))
   }
 
-  _npmInstall (program, codeDirectory) {
-    // Run on windows:
-    // https://nodejs.org/api/child_process.html#child_process_spawning_bat_and_cmd_files_on_windows
-
-    const installCommand = this._shouldUseNpmCi(codeDirectory)
-      ? 'ci'
-      : 'install'
-    const npmInstallBaseOptions = [
+  _getNpmInstallCommand (program, codeDirectory) {
+    const installOptions = [
       '-s',
-      installCommand,
+      this._shouldUseNpmCi(codeDirectory) ? 'ci' : 'install',
       '--production',
       '--no-audit'
     ]
 
     if (program.optionalDependencies === false) {
-      npmInstallBaseOptions.push('--no-optional')
+      installOptions.push('--no-optional')
     }
 
+    if (!program.dockerImage) {
+      installOptions.push('--prefix', codeDirectory)
+    }
+
+    return {
+      packageManager: 'npm',
+      installOptions
+    }
+  }
+
+  _getYarnInstallCommand (program, codeDirectory) {
+    const installOptions = [
+      '-s',
+      'install',
+      '--production'
+    ]
+
+    if (program.optionalDependencies === false) {
+      installOptions.push('--ignore-optional')
+    }
+
+    if (!program.dockerImage) {
+      installOptions.push('--cwd', codeDirectory)
+    }
+
+    return {
+      packageManager: 'yarn',
+      installOptions
+    }
+  }
+
+  _packageInstall (program, codeDirectory) {
+    // Run on windows:
+    // https://nodejs.org/api/child_process.html#child_process_spawning_bat_and_cmd_files_on_windows
+
+    const { packageManager, installOptions } = (() => {
+      // default npm
+      if (program.packageManager === 'yarn') {
+        return this._getYarnInstallCommand(program, codeDirectory)
+      }
+      return this._getNpmInstallCommand(program, codeDirectory)
+    })()
+
     const paramsOnContainer = (() => {
       // with docker
       let dockerVolumesOptions = []
       program.dockerVolumes && program.dockerVolumes.split(' ').forEach((volume) => {
         dockerVolumesOptions = dockerVolumesOptions.concat(['-v', volume])
       })
-      const dockerCommand = [program.dockerImage, 'npm'].concat(npmInstallBaseOptions)
+      const dockerCommand = [program.dockerImage, packageManager].concat(installOptions)
       const dockerBaseOptions = [
         'run', '--rm',
         '-v', `${fs.realpathSync(codeDirectory)}:/var/task`,
@@ -429,17 +466,16 @@ Emulate only the body of the API Gateway event.
     })()
 
     const paramsOnHost = (() => {
-      // simple npm install
-      const options = npmInstallBaseOptions.concat(['--prefix', codeDirectory])
+      // simple install
       if (process.platform === 'win32') {
         return {
           command: 'cmd.exe',
-          options: ['/c', 'npm'].concat(options)
+          options: ['/c', packageManager].concat(installOptions)
         }
       }
       return {
-        command: 'npm',
-        options
+        command: packageManager,
+        options: installOptions
       }
     })()
 
@@ -450,7 +486,7 @@ Emulate only the body of the API Gateway event.
         env: process.env
       }, (err) => {
         if (err) return reject(err)
-        resolve()
+        resolve(packageManager)
       })
     })
   }
@@ -518,7 +554,7 @@ Emulate only the body of the API Gateway event.
 
   _codeDirectory () {
     // Why realpathSync?:
-    // If tmpdir is symbolic link and npm>=7, `this._npmInstall()` may not work properly.
+    // If tmpdir is symbolic link and npm>=7, `this._packageInstall()` may not work properly.
     return path.join(fs.realpathSync(os.tmpdir()), `${path.basename(path.resolve('.'))}-lambda`)
   }
 
@@ -691,8 +727,9 @@ they may not work as expected in the Lambda environment.
     console.log('=> Moving files to temporary directory')
     await this._fileCopy(program, lambdaSrcDirectory, codeDirectory, true)
     if (!program.keepNodeModules) {
-      console.log(`=> Running npm ${this._shouldUseNpmCi(codeDirectory) ? 'ci' : 'install'} --production`)
-      await this._npmInstall(program, codeDirectory)
+      console.log('=> Running package install')
+      const usedPackageManager = await this._packageInstall(program, codeDirectory)
+      console.log(`(Package manager used was '${usedPackageManager}'.)`)
     }
     await this._postInstallScript(program, codeDirectory)
     console.log('=> Zipping deployment package')
